@@ -59,9 +59,11 @@ def rst_to_coq(coq, fpath, point, marker):
     from .literate import rst2coq_marked
     return _catch_parsing_errors(fpath, rst2coq_marked, coq, point, marker)
 
-def annotate_chunks(chunks, sertop_args):
-    from .core import annotate
-    return annotate(chunks, sertop_args)
+def annotate_chunks(chunks, fpath, cache_directory, sertop_args):
+    from .core import SerAPI, annotate
+    from .json import Cache
+    cache = Cache(cache_directory, fpath, sertop_args)
+    return cache.update(chunks, lambda c: annotate(c, sertop_args), SerAPI.version_info())
 
 def register_docutils(v, sertop_args):
     from .docutils import setup, AlectryonTransform
@@ -69,28 +71,25 @@ def register_docutils(v, sertop_args):
     setup()
     return v
 
-def _gen_docutils_html(source, fpath,
-                       webpage_style, include_banner, include_vernums,
-                       html_assets, traceback, Parser, Reader):
+def _gen_docutils(source, fpath,
+                  include_banner, include_vernums,
+                  traceback, Parser, Reader, Writer,
+                  settings_overrides):
     from docutils.core import publish_string
-    from .docutils import HtmlTranslator, HtmlWriter
 
     # The encoding/decoding dance below happens because setting output_encoding
     # to "unicode" causes reST to generate a bad <meta> tag, and setting
     # input_encoding to "unicode" breaks the ‘.. include’ directive.
 
-    html_assets.extend(HtmlTranslator.JS + HtmlTranslator.CSS)
-
     settings_overrides = {
         'traceback': traceback,
         'embed_stylesheet': False,
         'stylesheet_path': None,
-        'stylesheet_dirs': [],
         'alectryon_banner': include_banner,
         'alectryon_vernums': include_vernums,
-        'webpage_style': webpage_style,
         'input_encoding': 'utf-8',
-        'output_encoding': 'utf-8'
+        'output_encoding': 'utf-8',
+        **settings_overrides
     }
 
     parser = Parser()
@@ -99,31 +98,42 @@ def _gen_docutils_html(source, fpath,
         source_path=fpath, destination_path=None,
         reader=Reader(parser), reader_name=None,
         parser=parser, parser_name=None,
-        writer=HtmlWriter(), writer_name=None,
+        writer=Writer(), writer_name=None,
         settings=None, settings_spec=None,
         settings_overrides=settings_overrides, config_section=None,
         enable_exit_status=True).decode("utf-8")
 
-def gen_rstcoq_html(coq, fpath, webpage_style,
-                    include_banner, include_vernums,
-                    html_assets, traceback):
-    from .docutils import RSTCoqParser, RSTCoqStandaloneReader
-    return _gen_docutils_html(coq, fpath, webpage_style,
-                         include_banner, include_vernums,
-                         html_assets, traceback,
-                         RSTCoqParser, RSTCoqStandaloneReader)
+def gen_docutils(src, frontend, backend, fpath,
+                 webpage_style, include_banner, include_vernums,
+                 assets, traceback):
+    if frontend == "coq+rst":
+        from .docutils import RSTCoqParser as Parser
+        from .docutils import RSTCoqStandaloneReader as Reader
+    elif frontend == "rst":
+        from docutils.parsers.rst import Parser
+        from docutils.readers.standalone import Reader
+    else:
+        raise ValueError("Unsupported docutils frontend: {}".format(frontend))
 
-def gen_rst_html(rst, fpath, webpage_style,
-                 include_banner, include_vernums,
-                 html_assets, traceback):
-    from docutils.parsers.rst import Parser
-    from docutils.readers.standalone import Reader
-    return _gen_docutils_html(rst, fpath, webpage_style,
-                         include_banner, include_vernums,
-                         html_assets, traceback,
-                         Parser, Reader)
+    if backend == "webpage":
+        from .docutils import HtmlTranslator, HtmlWriter as Writer
+        assets.extend(HtmlTranslator.JS + HtmlTranslator.CSS)
+        settings_overrides = {"webpage_style": webpage_style}
+    elif backend in ("xelatex", "latex"):
+        if backend == "xelatex":
+            from .docutils import XeLatexTranslator as Translator, XeLatexWriter as Writer
+        else:
+            from .docutils import LatexTranslator as Translator, LatexWriter as Writer
+        assets.extend(Translator.STY)
+        settings_overrides = {}
+    else:
+        raise ValueError("Unsupported docutils backend: {}".format(backend))
 
-def _docutils_cmdline(description, Reader, Parser):
+    return _gen_docutils(src, fpath, include_banner, include_vernums,
+                         traceback, Parser, Reader, Writer,
+                         settings_overrides)
+
+def _docutils_cmdline_html(description, Reader, Parser):
     import locale
     locale.setlocale(locale.LC_ALL, '')
 
@@ -173,7 +183,12 @@ def _scrub_fname(fname):
     import re
     return re.sub("[^-a-zA-Z0-9]", "-", fname)
 
-def gen_html_snippets(annotated, include_vernums, fname):
+def apply_transforms(annotated):
+    from .transforms import default_transform
+    for chunk in annotated:
+        yield default_transform(chunk)
+
+def gen_html_snippets(annotated, fname):
     from .html import HtmlGenerator
     from .pygments import highlight_html
     return HtmlGenerator(highlight_html, _scrub_fname(fname)).gen(annotated)
@@ -209,7 +224,8 @@ def _gen_coqdoc_html(coqdoc_fragments):
     coqdoc_output = _run_coqdoc(fr.contents for fr in coqdoc_fragments)
     soup = BeautifulSoup(coqdoc_output, "html.parser")
     docs = soup.find_all(class_='doc')
-    if len(docs) != sum(1 for c in coqdoc_fragments if not c.special):
+    coqdoc_comments = [c for c in coqdoc_fragments if not c.special]
+    if len(docs) != len(coqdoc_comments):
         from pprint import pprint
         print("Coqdoc mismatch:", file=sys.stderr)
         pprint(list(zip(coqdoc_comments, docs)))
@@ -243,15 +259,27 @@ def gen_html_snippets_with_coqdoc(annotated, html_classes, fname):
     # ‘return’ instead of ‘yield from’ to update html_classes eagerly
     return _gen_html_snippets_with_coqdoc(annotated, fname)
 
-def copy_assets(state, html_assets, copy_fn, output_directory):
-    from .html import copy_assets as cp
-    if copy_fn:
-        cp(output_directory, assets=html_assets, copy_fn=copy_fn)
+def copy_assets(state, assets, copy_fn, output_directory):
+    from .html import ASSETS
+
+    for name in assets:
+        src = os.path.join(ASSETS.PATH, name)
+        dst = os.path.join(output_directory, name)
+        if copy_fn is not shutil.copy:
+            try:
+                os.unlink(dst)
+            except FileNotFoundError:
+                pass
+        try:
+            copy_fn(src, dst)
+        except shutil.SameFileError:
+            pass
+
     return state
 
 def dump_html_standalone(snippets, fname, webpage_style,
                          include_banner, include_vernums,
-                         html_assets, html_classes):
+                         assets, html_classes):
     from dominate import tags, document
     from dominate.util import raw
     from . import GENERATOR
@@ -274,8 +302,8 @@ def dump_html_standalone(snippets, fname, webpage_style,
     for js in ASSETS.ALECTRYON_JS:
         doc.head.add(tags.script(src=js))
 
-    html_assets.extend(ASSETS.ALECTRYON_CSS)
-    html_assets.extend(ASSETS.ALECTRYON_JS)
+    assets.extend(ASSETS.ALECTRYON_CSS)
+    assets.extend(ASSETS.ALECTRYON_JS)
 
     pygments_css = HTML_FORMATTER.get_style_defs('.highlight')
     doc.head.add(tags.style(pygments_css, type="text/css"))
@@ -330,54 +358,81 @@ def write_file(ext):
     return lambda contents, fname, output, output_directory: \
         write_output(ext, contents, fname, output, output_directory)
 
+# No ‘apply_transforms’ in JSON pipelines: (we save the prover output without
+# modifications).
 PIPELINES = {
     'json': {
-        'json': (read_json, annotate_chunks,
-                 prepare_json, dump_json, write_file(".io.json")),
-        'snippets-html': (read_json, annotate_chunks, gen_html_snippets,
-                          dump_html_snippets, write_file(".snippets.html")),
-        'snippets-latex': (read_json, annotate_chunks, gen_latex_snippets,
-                           dump_latex_snippets, write_file(".snippets.tex"))
+        'json':
+        (read_json, annotate_chunks,
+         prepare_json, dump_json, write_file(".io.json")),
+        'snippets-html':
+        (read_json, annotate_chunks, apply_transforms, gen_html_snippets,
+         dump_html_snippets, write_file(".snippets.html")),
+        'snippets-latex':
+        (read_json, annotate_chunks, apply_transforms, gen_latex_snippets,
+         dump_latex_snippets, write_file(".snippets.tex"))
     },
     'coq': {
-        'null': (read_plain, parse_coq_plain, annotate_chunks),
-        'webpage': (read_plain, parse_coq_plain, annotate_chunks,
-                    gen_html_snippets, dump_html_standalone, copy_assets,
-                    write_file(".v.html")),
-        'snippets-html': (read_plain, parse_coq_plain, annotate_chunks,
-                          gen_html_snippets, dump_html_snippets,
-                          write_file(".snippets.html")),
-        'snippets-latex': (read_plain, parse_coq_plain, annotate_chunks,
-                           gen_latex_snippets, dump_latex_snippets,
-                           write_file(".snippets.tex")),
-        'lint': (read_plain, register_docutils, lint_rstcoq,
-                 write_file(".lint.json")),
-        'rst': (read_plain, coq_to_rst, write_file(".v.rst")),
-        'json': (read_plain, parse_coq_plain, annotate_chunks, prepare_json,
-                 dump_json, write_file(".io.json"))
+        'null':
+        (read_plain, parse_coq_plain, annotate_chunks),
+        'webpage':
+        (read_plain, parse_coq_plain, annotate_chunks, apply_transforms,
+         gen_html_snippets, dump_html_standalone, copy_assets,
+         write_file(".v.html")),
+        'snippets-html':
+        (read_plain, parse_coq_plain, annotate_chunks, apply_transforms,
+         gen_html_snippets, dump_html_snippets, write_file(".snippets.html")),
+        'snippets-latex':
+        (read_plain, parse_coq_plain, annotate_chunks, apply_transforms,
+         gen_latex_snippets, dump_latex_snippets, write_file(".snippets.tex")),
+        'lint':
+        (read_plain, register_docutils, lint_rstcoq,
+         write_file(".lint.json")),
+        'rst':
+        (read_plain, coq_to_rst, write_file(".v.rst")),
+        'json':
+        (read_plain, parse_coq_plain, annotate_chunks, prepare_json,
+         dump_json, write_file(".io.json"))
     },
     'coq+rst': {
-        'webpage': (read_plain, register_docutils, gen_rstcoq_html, copy_assets,
-                    write_file(".html")),
-        'lint': (read_plain, register_docutils, lint_rstcoq,
-                 write_file(".lint.json")),
-        'rst': (read_plain, coq_to_rst, write_file(".v.rst"))
+        'webpage':
+        (read_plain, register_docutils, gen_docutils, copy_assets,
+         write_file(".html")),
+        'latex':
+        (read_plain, register_docutils, gen_docutils, copy_assets,
+         write_file(".tex")),
+        'lint':
+        (read_plain, register_docutils, lint_rstcoq,
+         write_file(".lint.json")),
+        'rst':
+        (read_plain, coq_to_rst, write_file(".v.rst"))
     },
     'coqdoc': {
-        'webpage': (read_plain, parse_coq_plain, annotate_chunks,
-                    gen_html_snippets_with_coqdoc, dump_html_standalone,
-                    copy_assets, write_file(".html")),
+        'webpage': # transforms applied later
+        (read_plain, parse_coq_plain, annotate_chunks,
+         gen_html_snippets_with_coqdoc, dump_html_standalone,
+         copy_assets, write_file(".html")),
     },
     'rst': {
-        'webpage': (read_plain, register_docutils, gen_rst_html, copy_assets,
-                    write_file(".html")),
-        'lint': (read_plain, register_docutils, lint_rst,
-                 write_file(".lint.json")),
-        'coq': (read_plain, rst_to_coq, write_file(".v")),
-        'coq+rst': (read_plain, rst_to_coq, write_file(".v"))
+        'webpage':
+        (read_plain, register_docutils, gen_docutils, copy_assets,
+         write_file(".html")),
+        'latex':
+        (read_plain, register_docutils, gen_docutils, copy_assets,
+         write_file(".tex")),
+        'lint':
+        (read_plain, register_docutils, lint_rst,
+         write_file(".lint.json")),
+        'coq':
+        (read_plain, rst_to_coq, write_file(".v")),
+        'coq+rst':
+        (read_plain, rst_to_coq, write_file(".v"))
     }
 }
 
+PIPELINES['rst']['xelatex'] = PIPELINES['rst']['latex']
+PIPELINES['coq+rst']['xelatex'] = PIPELINES['coq+rst']['latex']
+
 # CLI
 # ===
 
@@ -390,7 +445,8 @@ BACKENDS_BY_EXTENSION = [
     ('.lint.json', 'lint'),
     ('.snippets.html', 'snippets-html'),
     ('.snippets.tex', 'snippets-latex'),
-    ('.v.html', 'webpage'), ('.html', 'webpage')
+    ('.v.html', 'webpage'), ('.html', 'webpage'),
+    ('.v.tex', 'latex'), ('.tex', 'latex')
 ]
 
 DEFAULT_BACKENDS = {
@@ -418,12 +474,12 @@ def infer_backend(frontend, out_fpath):
     return DEFAULT_BACKENDS[frontend]
 
 def resolve_pipeline(fpath, args):
-    frontend = args.frontend or infer_frontend(fpath)
+    args.frontend = frontend = args.frontend or infer_frontend(fpath)
 
     assert frontend in PIPELINES
     supported_backends = PIPELINES[frontend]
 
-    backend = args.backend or infer_backend(frontend, args.output)
+    args.backend = backend = args.backend or infer_backend(frontend, args.output)
     if backend not in supported_backends:
         MSG = """argument --backend: Frontend {!r} does not support backend {!r}: \
 expecting one of {}"""
@@ -464,7 +520,7 @@ def post_process_arguments(parser, args):
             MSG = "argument --mark-point: Expecting a number, not {!r}"
             parser.error(MSG.format(args.point))
 
-    args.html_assets = []
+    args.assets = []
     args.html_classes = []
     args.pipelines = [(fpath, resolve_pipeline(fpath, args))
                       for fpath in args.input]
@@ -574,7 +630,7 @@ and produce reStructuredText, HTML, or JSON output.""")
     parser.add_argument("--expect-unexpected", action="store_true",
                         default=False, help=EXPECT_UNEXPECTED_HELP)
 
-    DEBUG_HELP = "Print communications with SerAPI."
+    DEBUG_HELP = "Print communications with prover process."
     parser.add_argument("--debug", action="store_true",
                         default=False, help=DEBUG_HELP)
 
@@ -612,11 +668,21 @@ def build_context(fpath, args):
 
     return ctx
 
+def except_hook(etype, value, tb):
+    from traceback import TracebackException
+    for line in TracebackException(etype, value, tb, capture_locals=True).format():
+        print(line, file=sys.stderr)
+
 def process_pipelines(args):
     if args.debug:
         from . import core
         core.DEBUG = True
 
+    if args.traceback:
+        from . import core
+        core.TRACEBACK = True
+        sys.excepthook = except_hook
+
     if args.cache_directory:
         from . import docutils
         docutils.CACHE_DIRECTORY = args.cache_directory
@@ -625,33 +691,33 @@ def process_pipelines(args):
         from . import core
         core.SerAPI.EXPECT_UNEXPECTED = True
 
+    for fpath, pipeline in args.pipelines:
+        state, ctx = None, build_context(fpath, args)
+        for step in pipeline:
+            state = call_pipeline_step(step, state, ctx)
+
+def main():
     try:
-        for fpath, pipeline in args.pipelines:
-            state, ctx = None, build_context(fpath, args)
-            for step in pipeline:
-                state = call_pipeline_step(step, state, ctx)
-    except (ValueError, FileNotFoundError) as e:
-        if args.traceback:
+        args = parse_arguments()
+        process_pipelines(args)
+    except (ValueError, FileNotFoundError, argparse.ArgumentTypeError) as e:
+        from . import core
+        if core.TRACEBACK:
             raise e
         print("Exiting early due to an error:", file=sys.stderr)
         print(str(e), file=sys.stderr)
         sys.exit(1)
 
-
-def main():
-    args = parse_arguments()
-    process_pipelines(args)
-
 # Alternative CLIs
 # ================
 
 def rstcoq2html():
     from .docutils import RSTCoqStandaloneReader, RSTCoqParser
     DESCRIPTION = 'Build an HTML document from an Alectryon Coq file.'
-    _docutils_cmdline(DESCRIPTION, RSTCoqStandaloneReader, RSTCoqParser)
+    _docutils_cmdline_html(DESCRIPTION, RSTCoqStandaloneReader, RSTCoqParser)
 
 def coqrst2html():
     from docutils.parsers.rst import Parser
     from docutils.readers.standalone import Reader
     DESCRIPTION = 'Build an HTML document from an Alectryon reStructuredText file.'
-    _docutils_cmdline(DESCRIPTION, Reader, Parser)
+    _docutils_cmdline_html(DESCRIPTION, Reader, Parser)
