@@ -52,6 +52,16 @@ which will set the default role to ``:coq:``.
 
 For convenience, ``alectryon.docutils.setup()`` can be used to perform all the
 steps above at once.
+
+A note on transforms: Sphinx has a nice API (``app.add_node``) for adding new
+node types, so you can write visitors for each output format without creating
+new translators.  Docutils doesn't have such an API: it forces you to subclass
+the default translator instead, which is a pain.  The alternative is to use a
+transform to replace custom nodes with "raw" nodes, but even this is non-trivial
+because the transform doesn't know which output format the document uses.
+
+To work around this issue we use a writer-dependent transform on the docutils
+side, and a doctree-resolved event on the Sphinx side.
 """
 
 import re
@@ -64,12 +74,13 @@ from docutils.parsers.rst import directives, roles, Directive
 from docutils.parsers.rst.directives.body import Sidebar
 from docutils.readers.standalone import Reader
 from docutils.transforms import Transform
-from docutils.writers import get_writer_class
+from docutils.writers import html4css1, latex2e, xetex
 
 from . import transforms
-from .core import annotate, SerAPI, GeneratorInfo
-from .html import ASSETS, ADDITIONAL_HEADS, HtmlGenerator, gen_banner, wrap_classes
-from .pygments import highlight_html, added_tokens, replace_builtin_coq_lexer
+from .core import annotate, SerAPI
+from .html import ADDITIONAL_HEADS, HtmlGenerator, gen_banner, wrap_classes, ASSETS as ASSETS_HTML
+from .latex import LatexGenerator, ASSETS as ASSETS_LATEX
+from .pygments import highlight_html, highlight_latex, added_tokens, replace_builtin_coq_lexer
 
 # reST extensions
 # ===============
@@ -86,6 +97,9 @@ class alectryon_pending(nodes.pending):
 class alectryon_pending_toggle(nodes.pending):
     pass
 
+class alectryon_io(docutils.nodes.Element):
+    pass
+
 # Transforms
 # ----------
 
@@ -151,7 +165,7 @@ class Config:
                 yield ",".join(vals)
 
 class AlectryonTransform(Transform):
-    default_priority = 995
+    default_priority = 990
     auto_toggle = True
 
     SERTOP_ARGS = ()
@@ -165,11 +179,6 @@ class AlectryonTransform(Transform):
                 fr.annots.inherit(annots)
             yield fr
 
-    @staticmethod
-    def document_id(document):
-        source = document.get('source', "")
-        return nodes.make_id(os.path.basename(source))
-
     def check_for_long_lines(self, node, fragments):
         if LONG_LINE_THRESHOLD is None:
             return
@@ -181,19 +190,16 @@ class AlectryonTransform(Transform):
     def annotate_cached(self, chunks, sertop_args):
         from .json import Cache
         cache = Cache(CACHE_DIRECTORY, self.document['source'], sertop_args)
-        annotated = cache.get(chunks)
-        if annotated is None:
-            # Later: decouple from .core by generalizing over `annotate`
-            annotated = annotate(chunks, sertop_args)
-            cache.put(chunks, annotated, SerAPI.version_info())
+        annotated = cache.update(chunks, lambda c: annotate(c, sertop_args), SerAPI.version_info())
         return cache.generator, annotated
 
-    def annotate(self, pending_nodes, config):
+    def annotate(self, pending_nodes):
+        config = Config(self.document)
         sertop_args = (*self.SERTOP_ARGS, *config.sertop_args)
         chunks = [pending.details["contents"] for pending in pending_nodes]
         return self.annotate_cached(chunks, sertop_args)
 
-    def replace_node(self, config, writer, pending, fragments):
+    def replace_node(self, pending, fragments):
         annots = transforms.IOAnnots(*pending.details['options'])
         if annots.hide:
             pending.parent.remove(pending)
@@ -201,19 +207,16 @@ class AlectryonTransform(Transform):
         fragments = self.set_fragment_annots(fragments, annots)
         fragments = transforms.default_transform(fragments)
         self.check_for_long_lines(pending, fragments)
-        with added_tokens(config.tokens):
-            html = writer.gen_fragments(fragments).render(pretty=False)
         contents = pending.details["contents"]
-        pending.replace_self(nodes.raw(contents, html, format='html'))
+        io = alectryon_io(fragments=fragments, contents=contents)
+        pending.replace_self(io)
 
     def apply_coq(self):
-        config = Config(self.document)
-        pending_nodes = self.document.traverse(alectryon_pending)
-        generator, annotated = self.annotate(pending_nodes, config)
+        pending_nodes = list(self.document.traverse(alectryon_pending))
+        generator, annotated = self.annotate(pending_nodes)
         _alectryon_state(self.document).generator = generator
-        writer = HtmlGenerator(highlight_html, gensym_stem=self.document_id(self.document))
         for node, fragments in zip(pending_nodes, annotated):
-            self.replace_node(config, writer, node, fragments)
+            self.replace_node(node, fragments)
 
     @staticmethod
     def split_around(node):
@@ -251,6 +254,35 @@ class AlectryonTransform(Transform):
             self.apply_coq()
             self.apply_toggle()
 
+class AlectryonPostTransform(Transform):
+    default_priority = 995
+
+    @staticmethod
+    def document_id(document):
+        source = document.get('source', "")
+        return nodes.make_id(os.path.basename(source))
+
+    def init_writer(self):
+        raise NotImplementedError("Unknown output format")
+
+    def apply(self, **_kwargs):
+        config = Config(self.document)
+        fmt, writer = self.init_writer()
+        with added_tokens(config.tokens):
+            for node in self.document.traverse(alectryon_io):
+                fragments, contents = node["fragments"], node["contents"]
+                raw = writer.gen_fragments(fragments).render(pretty=False)
+                node.replace_self(nodes.raw(contents, raw, format=fmt))
+
+class AlectryonHTMLPostTransform(AlectryonPostTransform):
+    def init_writer(self):
+        gensym_stem = self.document_id(self.document)
+        return "html", HtmlGenerator(highlight_html, gensym_stem=gensym_stem)
+
+class AlectryonLatexPostTransform(AlectryonPostTransform):
+    def init_writer(self):
+        return "latex", LatexGenerator(highlight_latex)
+
 # Directives
 # ----------
 
@@ -454,9 +486,6 @@ class RSTCoqParser(docutils.parsers.rst.Parser):
     """A wrapper around the reStructuredText parser for literate Coq files."""
 
     supported = ('coq',)
-    """Aliases this parser supports."""
-    # settings_spec = docutils.parsers.rst.Parser.settings_spec + \
-    #     ('Literate Coq Parser Options', None, ())
     config_section = 'Literate Coq parser'
     config_section_dependencies = ('parsers',)
 
@@ -494,9 +523,9 @@ class RSTCoqParser(docutils.parsers.rst.Parser):
         self.setup_parse(inputstring, document)
         # pylint: disable=attribute-defined-outside-init
         self.statemachine = docutils.parsers.rst.states.RSTStateMachine(
-              state_classes=self.state_classes,
-              initial_state=self.initial_state,
-              debug=document.reporter.debug_flag)
+            state_classes=self.state_classes,
+            initial_state=self.initial_state,
+            debug=document.reporter.debug_flag)
         try:
             lines = RSTCoqParser.coq_input_lines(inputstring, document['source'])
             self.statemachine.run(lines, document, inliner=self.inliner)
@@ -518,30 +547,31 @@ class RSTCoqStandaloneReader(Reader):
 # Writer
 # ------
 
-DefaultWriter = get_writer_class('html')
+def register_stylesheets(translator, stylesheets):
+    for name in stylesheets:
+        if translator.settings.embed_stylesheet:
+            # Expand only if we're going to inline; otherwise keep relative
+            name = os.path.join(ASSETS_HTML.PATH, name)
+        translator.stylesheet.append(translator.stylesheet_call(name))
 
-class HtmlTranslator(DefaultWriter().translator_class):
-    JS = ASSETS.ALECTRYON_JS
-    CSS = (*ASSETS.ALECTRYON_CSS, *ASSETS.DOCUTILS_CSS, *ASSETS.PYGMENTS_CSS)
-    ADDITIONAL_HEADS = [ASSETS.IBM_PLEX_CDN, ASSETS.FIRA_CODE_CDN, *ADDITIONAL_HEADS]
+class HtmlTranslator(html4css1.HTMLTranslator): \
+      # pylint: disable=abstract-method
+    JS = ASSETS_HTML.ALECTRYON_JS
+    CSS = (*ASSETS_HTML.ALECTRYON_CSS, *ASSETS_HTML.DOCUTILS_CSS, *ASSETS_HTML.PYGMENTS_CSS)
+    ADDITIONAL_HEADS = [ASSETS_HTML.IBM_PLEX_CDN, ASSETS_HTML.FIRA_CODE_CDN, *ADDITIONAL_HEADS]
 
     JS_TEMPLATE = '<script type="text/javascript" src="{}"></script>\n'
     MATHJAX_URL = "https://cdnjs.cloudflare.com/ajax/libs/mathjax/3.1.2/es5/tex-mml-chtml.min.js"
 
-    head_prefix_template = ('<html xmlns="http://www.w3.org/1999/xhtml" class="alectryon-standalone"'
-                            ' xml:lang="%(lang)s" lang="%(lang)s">\n<head>\n')
-
-    def stylesheet_call(self, name):
-        if self.settings.embed_stylesheet:
-            # Expand only if we're going to inline; otherwise keep relative
-            name = os.path.join(ASSETS.PATH, name)
-        return super().stylesheet_call(name)
+    head_prefix_template = \
+        '<html xmlns="http://www.w3.org/1999/xhtml" class="alectryon-standalone"' \
+        ' xml:lang="%(lang)s" lang="%(lang)s">\n<head>\n'
 
     def __init__(self, document):
         document.settings.syntax_highlight = "short"
         document.settings.math_output = "MathJax " + self.MATHJAX_URL
         super().__init__(document)
-        self.stylesheet.extend(self.stylesheet_call(css) for css in self.CSS)
+        register_stylesheets(self, self.CSS)
         self.stylesheet.extend(self.JS_TEMPLATE.format(js) for js in self.JS)
         self.stylesheet.extend(hd + "\n" for hd in self.ADDITIONAL_HEADS)
         cls = wrap_classes(self.settings.webpage_style)
@@ -552,8 +582,8 @@ class HtmlTranslator(DefaultWriter().translator_class):
             self.body_prefix.append(gen_banner(generator, include_vernums))
         self.body_suffix.insert(0, '</div>')
 
-ALECTRYON_SETTINGS = [
-    ("Choose an Alectryon style",
+ALECTRYON_SETTINGS = (
+    ("Choose an Alectryon webpage style",
      ["--webpage-style"],
      {"choices": ("centered", "floating", "windowed"),
       "dest": "webpage_style",
@@ -568,17 +598,46 @@ ALECTRYON_SETTINGS = [
      {'default': True, 'action': 'store_false',
       'dest': "alectryon_vernums",
       'validator': frontend.validate_boolean})
-]
+)
 
-class HtmlWriter(DefaultWriter):
-    settings_spec = ('HTML-Specific Options', None,
-                     (*ALECTRYON_SETTINGS,
-                      *DefaultWriter.settings_spec[-1]))
+class HtmlWriter(html4css1.Writer):
+    settings_spec = (html4css1.Writer.settings_spec +
+                     ('Alectryon HTML writer options',
+                      None, ALECTRYON_SETTINGS))
+
+    def get_transforms(self):
+        return super().get_transforms() + [AlectryonHTMLPostTransform]
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
         self.translator_class = HtmlTranslator
 
+def make_LatexTranslator(base):
+    class Translator(base): \
+          # pylint: disable=abstract-method
+        STY = ASSETS_LATEX.ALECTRYON_STY + ASSETS_LATEX.PYGMENTS_STY
+
+        def __init__(self, document, *args, **kwargs):
+            super().__init__(document, *args, **kwargs)
+            register_stylesheets(self, self.STY)
+    return Translator
+
+LatexTranslator = make_LatexTranslator(latex2e.LaTeXTranslator)
+XeLatexTranslator = make_LatexTranslator(xetex.XeLaTeXTranslator)
+
+def make_LatexWriter(base, translator_class):
+    class Writer(base):
+        def get_transforms(self):
+            return super().get_transforms() + [AlectryonLatexPostTransform]
+
+        def __init__(self, *args, **kwargs):
+            super().__init__(*args, **kwargs)
+            self.translator_class = translator_class
+    return Writer
+
+LatexWriter = make_LatexWriter(latex2e.Writer, LatexTranslator)
+XeLatexWriter = make_LatexWriter(xetex.Writer, XeLatexTranslator)
+
 # Entry points
 # ============
 
